<?php

namespace Drupal\blackbaud_sky_api;

use GuzzleHttp\Client;
use GuzzleHttp\Exception\TransferException;

/**
 * Class Blackbaud.
 *
 * @package Drupal\blackbaud_sky_api
 */
abstract class Blackbaud {

 /**
  * The Redirect URI.
  *
  * @var string
  */
  protected $redirect_uri;

 /**
  * The url we are checking.
  *
  * @var string
  */
  protected $url;

  /**
   * The Request Response.
   *
   * @var GuzzleHttp\Client
   */
  protected $request;

  /**
   * Constructor.
   */
  public function __construct() {
    // Sets up the redirect uri for later use.
    global $base_url;
    $base = $base_url;

    // Make sure we are on https.
    if (strpos($base, 'http:') !== FALSE) {
      $base = str_replace('http:', 'https:', $base);
    }

    // Set the full uri.
    $this->redirect_uri = $base . '/' . variable_get('blackbaud_sky_api_redirect_uri', '');
  }

  /**
   * Set the url we are checking.
   *
   * @param string $url
   *   The url string we are checking.
   */
  protected function setURL($url) {
    $this->url = $url;
  }

  /**
   * Get the url we are checking.
   *
   * @return string
   *   The url string we are checking.
   */
  protected function getURL() {
    return $this->url;
  }

  /**
   * Set the url we are checking.
   *
   * @param string $type
   *   The type of token we are setting.
   *
   * @param string $token
   *   The token we are setting.
   */
  protected function setToken($type, $token) {
    variable_set('blackbaud_sky_api_' . $type . '_token', $token);
  }

  /**
   * Get the url we are checking.
   *
   * @return string
   *   The type of token we are getting.
   */
  protected function getToken($type) {
    return variable_get('blackbaud_sky_api_' . $type . '_token', '');
  }

  /**
   * Gets the response of a page.
   *
   * @param string $type
   *   The type of request (ie GET, POST, etc).
   *
   * @param array $options
   *   Any options to pass in.
   */
  protected function requestResponse($type, $options = array()) {
    $client = new Client();

    // Set the options for the request.
    // @see http://docs.guzzlephp.org/en/latest/request-options.html
    $options += array(
      'http_errors' => FALSE,
      'timeout' => 3,
      'connect_timeout' => 3,
      'synchronous' => TRUE,
    );

    try {
      // Try the request.
      $response = $client->request($type, $this->getURL(), $options);

      // Check the Status code and return.
      switch ($response->getStatusCode()) {
        // All good, send back response.
        case '200':
          $this->request = $response;
          break;

        // Need to reauth.
        case '401':
          $this->getAuthCode('refresh', $this->getToken('refresh'));
          break;

        // Something else is amiss.
        default:
          $message = 'The request to the Blackbaud API resulted in a ' . $response->getStatusCode() . ' Response.';
          watchdog('Blackbaud SKY API', $message);
          $this->request = NULL;
          break;
      }
    }
    catch (TransferException $e) {
      $this->request = NULL;
    }
  }
}
