<?php

namespace Drupal\blackbaud_sky_api;

use Drupal\blackbaud_sky_api\Blackbaud;

/**
 * Class BlackbaudOauth.
 *
 * @package Drupal\blackbaud_sky_api
 */
class BlackbaudOauth extends Blackbaud {

  /**
   * Get the Oauth base URL we are checking.
   *
   * @return string
   *   The OAuth base URL.
   */
  public function getOAuthBaseURL() {
    return variable_get('blackbaud_sky_api_oauth_url', BLACKBAUD_SKY_API_OAUTH_URL);
  }

  /**
   * Gets the Code token via user interaction on a redirect.
   */
  public function getCode() {
    // Auth Url
    $this->setURL($this->getOAuthBaseURL() . '/authorization');

    // Set the auth query params.
    $query = array(
      'query' => array(
        'client_id' => variable_get('blackbaud_sky_api_application_id', ''),
        'response_type' => 'code',
        'redirect_uri' => $this->redirect_uri,
      ),
    );

    // The Url used for auth.
    $url = url(parent::getURL(), $query);

    // Go to the url and return to the redirect URI.
    drupal_goto($url);
  }

  /**
   * Get the Authorization code.
   *
   * @param string $type
   *   The type of Auth Code Request (init or refresh).
   *
   * @param string $code
   *   The $_GET param from the Oauth Callback page.
   */
  public function getAuthCode($type, $code) {
    // Access Token Url
    parent::setURL($this->getOAuthBaseURL() . '/token');

    // Grab these for the auth header.
    $client_id = variable_get('blackbaud_sky_api_application_id', '');
    $client_secret = variable_get('blackbaud_sky_api_application_secret', '');

    // Set header for this post request.
    $header = array(
      'Content-type' => 'application/x-www-form-urlencoded',
      'Authorization' => 'Basic ' . base64_encode($client_id . ':' . $client_secret)
    );

    // Set body for the type of post request.
    switch ($type) {
      // Initial Request.
      case 'init':
        $body = array(
          'grant_type' => 'authorization_code',
          'code' => $code,
          'redirect_uri' => $this->redirect_uri,
        );
        break;

      // Refresh Request.
      case 'refresh':
        $body = array(
          'grant_type' => 'refresh_token',
          'refresh_token' => $code,
        );
        break;
    }

    // Set the options.
    $options = array(
      'headers' => $header,
      'body' => http_build_query($body),
    );

    // Post the Data.
    parent::requestResponse('POST', $options);

    // Decode and grab the response.
    $contents = json_decode($this->request->getBody()->getContents());

    // Set the refresh token for later use.
    parent::setToken('refresh', $contents->refresh_token);

    // Set the access token.
    parent::setToken('access', $contents->access_token);

    // What to do after we set this up.
    switch ($type) {
      // Initial Request.
      case 'init':
        // Go back to authorization page
        drupal_goto('admin/config/services/blackbaud/authorize');
        break;

      // Refresh Request.
      case 'refresh':
        return TRUE;
        break;
    }

  }

}
